#include "StdAfx.h"
#include "ReceivedNoteManager.h"
#include "SendNoteMananger.h"
#include "CommonFunction.h"

ReceivedNoteManager::ReceivedNoteManager(void)
: m_nMaxDlgCount(30)
, m_pSQLite(NULL)
{
	InitSqlite();
}

ReceivedNoteManager::~ReceivedNoteManager(void)
{
	if (NULL != g_pMessengerActor)
    {
		g_pMessengerActor->GetWASActor()->UnRegisterObserver(this);	
    }
    CloseSqlite();
}

void ReceivedNoteManager::SetReadOnly( BOOL bStatus, CObject* pObject )
{
	CRecvMemoDlg* pDialog = (CRecvMemoDlg*) pObject;

	pDialog->SetReadOnly(bStatus);
}

bool ReceivedNoteManager::MakeMemoRecvDlg(UINT64 noteID)
{
	g_pMessengerActor->GetWASActor()->RequestGetRecvNoteInfo(noteID, reqNoteInfoType_MakeRecvDlg, GetLanguageString().c_str(), false);

	return true;
}

bool ReceivedNoteManager::MakeMemoRecvDlg(UINT64 noteID, const NoteInfo& noteInfo,bool flashEffect)
{
	CRecvMemoDlg* pDialog = new CRecvMemoDlg(noteID, noteInfo);
	if (NULL == pDialog)
		return false;

	if (GET_CONFIG()->GetNoteRecoveryType())
	{
		InsertRecvNoteToSqlite((ULONG)noteID);
	}
	

	pDialog->Create();
	if (pDialog->IsInitializeSuccess())
	{
		m_listDialog.AddTail((CObject*)pDialog);

		int nAlarmtype = GET_CONFIG()->GetMemoAlarmType();
		if (3 != nAlarmtype)
		{
			if (nAlarmtype == 1)
			{
				pDialog->ShowWindow(SW_SHOWMINNOACTIVE);
				pDialog->EnableWindow(TRUE);
			}
			else
			{
				pDialog->ShowWindow(SW_SHOWNOACTIVATE);
				pDialog->EnableWindow(TRUE);
			}
			
		    pDialog->DisplayFlashEffect();
			
		}
	}
	else
	{
		delete pDialog;
		pDialog = NULL;
		ASSERT(FALSE);
	}

	return true;
}

CRecvMemoDlg * ReceivedNoteManager::GetReceivedNoteDlg(UINT64 lMemoId)
{
	for(POSITION pos = m_listDialog.GetHeadPosition(); pos != NULL;)
	{
		CRecvMemoDlg* pDialog =  (CRecvMemoDlg*)m_listDialog.GetNext(pos);
		if(pDialog->GetNoteID() ==lMemoId)
		{
			return (CRecvMemoDlg*)pDialog;
		}
	}
	return NULL;
}

void ReceivedNoteManager::SetDlgRestore(UINT64 lMemoID)
{
	CRecvMemoDlg* pDialog = GetReceivedNoteDlg(lMemoID);
	if (pDialog!=NULL)
	{
		pDialog->ShowWindow(SW_RESTORE);
	}
}

int ReceivedNoteManager::DlgSpaceCount()
{
	return m_nMaxDlgCount-m_listDialog.GetCount() ;
}

void ReceivedNoteManager::SetViewLanguagePack()
{
	for(POSITION pos = m_listDialog.GetHeadPosition(); pos != NULL;)
	{
		CRecvMemoDlg* pDialog =  (CRecvMemoDlg*)m_listDialog.GetNext(pos);
		if(pDialog->GetSafeHwnd())
		{
			pDialog->__ResourceLoad();
		}
	}
}

void ReceivedNoteManager::Remove( CDialog* pDialog )
{
	GET_APP()->m_pSendNoteManager->RecvMemoDlgIsClosed(pDialog);

	DialogManager::Remove(pDialog);
}

bool ReceivedNoteManager::OnGTWASActor_ResponseNoteSetReadStatus(IGTWASActor *pActor, trid_t trid, GTWASCallErrorCode eErrorCode)
{
	for(POSITION pos = m_listDialog.GetHeadPosition(); pos != NULL;)
	{
		CRecvMemoDlg* pDialog =  (CRecvMemoDlg*)m_listDialog.GetNext(pos);
		if(pDialog->OnResponseNoteSetReadStatus(pActor, trid, eErrorCode) == false)
			break;
	}

	return false;
}

bool ReceivedNoteManager::OnGTWASActor_ResponseDeleteNote(IGTWASActor *pActor, trid_t trid, GTWASCallErrorCode eErrorCode, bool bResult)
{
	for(POSITION pos = m_listDialog.GetHeadPosition(); pos != NULL;)
	{
		CRecvMemoDlg* pDialog =  (CRecvMemoDlg*)m_listDialog.GetNext(pos);
		if(pDialog->OnResponseDeleteNote(pActor, trid, eErrorCode, bResult) == false)
			break;
	}

	return false;
}

int ReceivedNoteManager::InitSqlite(void)
{
	// SQLite
	CString strSQLiteFilePath = GetAppDataNtalkDir() + _T("\\Ntalkbiz.db");

	bool bGeneratedDB = false;
	if ( PathFileExists(strSQLiteFilePath) != TRUE )
		bGeneratedDB = true;

	int nRet = sqlite3_open( UnicodeToUTF8(strSQLiteFilePath).c_str(), &m_pSQLite);
	if( nRet == SQLITE_OK ) 
	{
		if( bGeneratedDB )
		{
			sqlite3_stmt* pVM;
			char *query = "CREATE TABLE my_recv_note(noteid UNSIGNED BIG INT UNIQUE NOT NULL, userid VARCHAR(24) NOT NULL DEFAULT '')";
			sqlite3_prepare(m_pSQLite, query, -1, &pVM, NULL);

			nRet = 0;
			nRet = sqlite3_step(pVM);
		}
	}
	else
	{
		AfxMessageBox(_T("Failed to initialize database for nnote"));
		m_pSQLite = NULL;
	}

	return 0;
}

int ReceivedNoteManager::CloseSqlite(void)
{
	if( m_pSQLite != NULL )
	{
		int ret = sqlite3_close(m_pSQLite);
        m_pSQLite = NULL;
	}

	return 0;
}

int ReceivedNoteManager::RestoreRecvNote(void)
{
	if( m_pSQLite == NULL )
		return 0;

	string strUserID = UnicodeToUTF8( g_pMessengerActor->GetUserID() );

	sqlite3_stmt* pVM;
	char query[1024];
	sprintf_s(query, 1024, "SELECT * FROM my_recv_note WHERE userid = ?"/*, strUserID.c_str()*/);
	sqlite3_prepare(m_pSQLite, query, -1, &pVM, NULL);
	sqlite3_reset(pVM);

	sqlite3_bind_text(pVM, 1, strUserID.c_str(), strUserID.length(), SQLITE_STATIC);

	int ret = sqlite3_step(pVM);
	if( ret != SQLITE_DONE )
	{
		while(1)
		{
			UINT64 noteID = sqlite3_column_int64(pVM, 0);
			GET_APP()->m_pReceivedNoteManager->MakeMemoRecvDlg(noteID);

			ret = sqlite3_step(pVM);
			if( ret != SQLITE_ROW )
				break;
		}
	}

	return 0;
}

int ReceivedNoteManager::DeleteRecvNoteFromSqlite(unsigned long noteID)
{
	sqlite3_stmt* pVM;
	char query[1024];
	sprintf_s(query, 1024, "DELETE FROM my_recv_note WHERE noteid = ?"/*, noteID*/);

	sqlite3_prepare(m_pSQLite, query, -1, &pVM, NULL);
	sqlite3_reset(pVM);
	sqlite3_bind_int64(pVM, 1, noteID);

	int ret = sqlite3_step(pVM);

	return 0;
}

int ReceivedNoteManager::DeleteAllRecvNoteFromSqlite()
{
	string strUserID;
	strUserID = UnicodeToUTF8(g_pMessengerActor->GetUserID());

	sqlite3_stmt* pVM;
	char query[1024];
	sprintf_s(query, 1024, "DELETE FROM my_recv_note WHERE userid = ?");

	sqlite3_prepare(m_pSQLite, query, -1, &pVM, NULL);
	sqlite3_reset(pVM);
	sqlite3_bind_text(pVM, 1, strUserID.c_str(), strUserID.length(), SQLITE_STATIC);

	int ret = sqlite3_step(pVM);

	return 0;
}

int ReceivedNoteManager::InsertRecvNoteToSqlite(unsigned long noteID)
{
	string strUserID;
	strUserID = UnicodeToUTF8(g_pMessengerActor->GetUserID());

	sqlite3_stmt* pVM;
	char query[1024];
	sprintf_s(query, 1024, "INSERT INTO my_recv_note (noteid, userid) VALUES (?, ?)"/*, noteID, strUserID.c_str()*/);
	sqlite3_prepare(m_pSQLite, query, -1, &pVM, NULL);
	sqlite3_reset(pVM);
	sqlite3_bind_int64(pVM, 1, noteID);
	sqlite3_bind_text(pVM, 2, strUserID.c_str(), strUserID.length(), SQLITE_STATIC);

	int ret = sqlite3_step(pVM);

	return 0;
}

int ReceivedNoteManager::InsertAllRecvNoteToSqlite()
{
	for(POSITION pos = m_listDialog.GetHeadPosition(); pos != NULL;)
	{
		CRecvMemoDlg* pDialog =  (CRecvMemoDlg*)m_listDialog.GetNext(pos);
		InsertRecvNoteToSqlite( (ULONG)pDialog->GetNoteID() );
	}

	return 0;
}
